GtkScrollbar#hscrollbar,
GtkScrollbar#vscrollbar {
  -GtkScrollbar-has-backward-stepper: 0;
  -GtkScrollbar-has-secondary-backward-stepper: 0;
  -GtkScrollbar-has-forward-stepper: 0;
  -GtkScrollbar-has-secondary-forward-stepper: 0;
}
  
